@@ -7,18 +7,18 @@ This module is in charge of data serialization / deserialization to:
     * ConfigObj format
 """
 
-import configparser
 import logging
 import os
 import re
-from collections import defaultdict
 from io import StringIO
-from zipfile import ZipFile
+import warnings
 
 import numpy as np
 import pandas as pd
 from pandas.testing import assert_frame_equal
 
+warnings.filterwarnings("ignore", category=UserWarning, module="openpyxl")
+
 try:
     from configobj import ConfigObj
 
@@ -50,6 +50,12 @@ PD_SENSIBLE_TESTING = {
 }
 
 
+def drop_internal_columns(df):
+    df = df.copy()
+    df = df[[c for c in df.columns if not c.startswith("__")]]
+    return df
+
+
 # =============================================================================
 # functional read() methods
 # =============================================================================
@@ -86,6 +92,10 @@ def read_data(fpath, sheet_name=None, engine=DEFAULT_EXCEL_ENGINE):
 def read_excel(fpath, sheet_name=None, engine=DEFAULT_EXCEL_ENGINE):
     """read excel file using pandas"""
     df_dict = pd.read_excel(fpath, sheet_name=sheet_name, engine=engine)
+    for tabname, df in df_dict.items():
+        # since first row is used as columns headers, index 0 is XLSX row#2
+        df_dict[tabname]["__rownb_xlsx"] = df.index + 2
+        pass
     # remove any blank lines
     ret = {k: df.dropna(how="all") for k, df in df_dict.items()}
     # also clean Unnamed columns
@@ -116,7 +126,9 @@ def read_configobj(fpath, as_content=False, sheet_name=None, row_prefix="sniff")
     else:
         row_prefix = ROW_PREFIX
     for tabname, data in config.items():
-        ret[tabname] = _cfg_section_to_df(tabname, data, len(row_prefix))
+        df = _cfg_section_to_df(tabname, data, len(row_prefix))
+        df["__rownb_xlsx"] = df.index + 2
+        ret[tabname] = df
     # filter out sheet_name
     if sheet_name is None:
         return ret
@@ -232,6 +244,7 @@ class IOMixin:
     def to_excel(self, fpath):
         with pd.ExcelWriter(fpath) as writer:
             for tabname, df in self._data.items():
+                df = drop_internal_columns(df)
                 df.to_excel(writer, sheet_name=tabname, index=False)
         return fpath
 
@@ -258,6 +271,7 @@ class IOMixin:
         if not fpath:
             config.filename = None
         for tabname, df in self._data.items():
+            df = drop_internal_columns(df)
             try:
                 units = self.getunits(tabname)
             except KeyError:
