@@ -735,11 +735,12 @@ class ValidatorMixin:
         result = schema.validate_dataframe(self._data[tabname])
         return result
 
-    def validate(self, row_reporting_offset="data"):
+    def validate(self, row_reporting_offset="data", readonly=False):
         """
         iterate through all tabs and validate eachone
 
-        row_reporting_offset can be used to offset rows reporting
+        `row_reporting_offset` can be used to offset rows reporting
+        `readonly` is mainly usefull for testing purpose, to chain validation
         """
         # keep initial data before processing them
         if not hasattr(self, "_raw_data"):
@@ -826,22 +827,30 @@ class ValidatorMixin:
             offset_ret = defaultdict(dict)
             for tabname, report in ret.items():
                 is_units = len(self.getunits(tabname)) > 0
-                offset = _get_rows_offset(row_reporting_offset, is_units)
+                # iterate over each rows
+                _data = self._data[tabname]
                 for index, _ in report.items():
                     try:
                         (colname, rownb) = index
                         if rownb in (None, "?"):
                             raise ValueError("switch...")
                     except:
-                        offset_ret[tabname][index] = report[
-                            index
-                        ]  # "general" section, without row clue
+                        # "general" section, without row clue
+                        offset_ret[tabname][index] = report[index]
                     else:
-                        offset_ret[tabname][(colname, rownb + offset)] = report[
-                            (colname, rownb)
-                        ]
-
+                        msgs = report[(colname, rownb)]
+                        # row_reporting_offset == "spreadsheet"
+                        reported_rownb = _data.loc[rownb]["__rownb_xlsx"]
+                        if row_reporting_offset == "data":
+                            if is_units:
+                                reported_rownb -= 2
+                            else:
+                                reported_rownb -= 1
+                        offset_ret[tabname][(colname, reported_rownb)] = msgs
             ret = offset_ret
+        if readonly:
+            # revert to initial dataframe
+            self._data = {tabname: df.copy() for tabname, df in self._raw_data.items()}
         return dict(ret)
 
     def dump_template(self):
@@ -861,23 +870,6 @@ class ValidatorMixin:
         return dic
 
 
-def _get_rows_offset(row_reporting_offset, is_units):
-    """by default (row_reporting_offset = 0),
-    row#0 means row#:
-                   | Data 0-based |Data 1-based | Spreadsheet  |
-    with units     | 0            |1            | 3            |
-    witthout units | 0            |1            | 2            |
-
-    """
-    if isinstance(row_reporting_offset, int):
-        offset = row_reporting_offset
-    elif row_reporting_offset == "data":
-        offset = 1  # equivalent to row_reporting_offset = 1
-    elif row_reporting_offset == "spreadsheet":
-        offset = 3 if is_units else 2
-    return offset
-
-
 if __name__ == "__main__":
     import doctest
 
