@@ -14,6 +14,7 @@ import pytest
 from gridsource import Data as IVData
 from gridsource import ValidData as VData
 from gridsource.validation import DataFrameSchematizer, ureg
+from gridsource.io import drop_internal_columns
 
 
 def test_DFS_00_schema_bulding():
@@ -364,17 +365,11 @@ def test_IVData_02(datadir):
     data = IVData()
     data.read_excel(os.path.join(indir, "test_00.xlsx"))
     data.read_schema(os.path.join(indir, "test_00.schema2.yaml"))
-    # default row reporting as "data"
-    ret = data.validate()
+    ret = data.validate("data")
     assert ret == {
         "cars": {("Year", 3): ["None is not of type 'integer'"]},
         "names": {"general": ["'name' is a required property"]},
     }
-    ret = data.validate(10)
-    assert ret == {
-        "cars": {("Year", 12): ["None is not of type 'integer'"]},
-        "names": {"general": ["'name' is a required property"]},
-    }
     ret = data.validate("spreadsheet")
     assert ret == {
         "cars": {("Year", 4): ["None is not of type 'integer'"]},
@@ -451,6 +446,7 @@ def test_IVData_04(datadir):
             "distA": {0: 0.0010000000000000005, 1: 0.002000000000000001},
             "distB": {0: 14.3, 1: 15.0},
             "Volume": {0: 1.5064562386943998, 1: 0.8183568665087998},
+            "__rownb_xlsx": {0: 3, 1: 4},
         }
     )
     actual_geom = data._data["geom"]
@@ -557,7 +553,7 @@ def test_IVData_05_failing_units(datadir):
     }
 
     assert data._schemas["volume"].build() == expected_schema
-    ret = data.validate("data")
+    ret = data.validate("data", readonly=True)
     expected_report = {
         "mass": {
             ("test_mass_g", 3): ["None is not of type 'number'"],
@@ -566,6 +562,15 @@ def test_IVData_05_failing_units(datadir):
         "volume": {"general": ["'test_volume_foot3' is a required property"]},
     }
     assert expected_report == ret
+    ret = data.validate("spreadsheet")
+    expected_report = {
+        "mass": {
+            ("test_mass_g", 5): ["None is not of type 'number'"],
+            ("test_mass_lbm", None): ["undefined units 'lbm'"],
+        },
+        "volume": {"general": ["'test_volume_foot3' is a required property"]},
+    }
+    assert expected_report == ret
 
 
 def test_IVData_06_units(datadir):
@@ -664,6 +669,7 @@ def test_IVData_06_units(datadir):
             "test_dist_foot": 2000,
             "test_string": "a",
             "groucho": 3.2,
+            "__rownb_xlsx": 3,
         },
         name=9.0,
     )
@@ -806,6 +812,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
             },
         }
     ).T
+    df = drop_internal_columns(df)
     expected_df = expected_df[[c for c in df]]
     assert pd.testing.assert_frame_equal(df, expected_df, check_dtype=False) is None
 
